@@ -152,13 +152,13 @@ def parse_time(string):
         return None
     try:
         return mod_datetime.datetime.strptime(string, mod_gpx.DATE_FORMAT)
-    except Exception as e:
-        if mod_re.match('^.*\.\d+Z$', string):
-            string = mod_re.sub('\.\d+Z', 'Z', string)
+    except ValueError as e:
+        if mod_re.match(r'^.*\.\d+Z$', string):
+            string = mod_re.sub(r'\.\d+Z', 'Z', string)
         try:
             return mod_datetime.datetime.strptime(string, mod_gpx.DATE_FORMAT)
-        except Exception as e:
-            mod_logging.error('Invalid timestemp %s' % string)
+        except ValueError as e:
+            mod_logging.error('Invalid timestamp %s' % string)
             return None
 
 
@@ -225,7 +225,7 @@ class GPXParser:
     def __parse_dom(self):
 
         node = self.xml_parser.get_first_child(name='gpx')
-        if not node:
+        if node is None:
             raise mod_gpx.GPXException('Document must have a `gpx` root node.')
 
         for node in self.xml_parser.get_children(node):
@@ -291,7 +291,7 @@ class GPXParser:
         lon = mod_utils.to_number(lon)
 
         elevation_node = self.xml_parser.get_first_child(node, 'ele')
-        elevation = mod_utils.to_number(self.xml_parser.get_node_data(elevation_node), 0)
+        elevation = mod_utils.to_number(self.xml_parser.get_node_data(elevation_node), None)
 
         time_node = self.xml_parser.get_first_child(node, 'time')
         time_str = self.xml_parser.get_node_data(time_node)
@@ -359,7 +359,7 @@ class GPXParser:
         lon = mod_utils.to_number(lon)
 
         elevation_node = self.xml_parser.get_first_child(node, 'ele')
-        elevation = mod_utils.to_number(self.xml_parser.get_node_data(elevation_node), 0)
+        elevation = mod_utils.to_number(self.xml_parser.get_node_data(elevation_node), None)
 
         time_node = self.xml_parser.get_first_child(node, 'time')
         time_str = self.xml_parser.get_node_data(time_node)
@@ -438,7 +438,7 @@ class GPXParser:
         time = parse_time(time_str)
 
         elevation_node = self.xml_parser.get_first_child(node, 'ele')
-        elevation = mod_utils.to_number(self.xml_parser.get_node_data(elevation_node), 0)
+        elevation = mod_utils.to_number(self.xml_parser.get_node_data(elevation_node), None)
 
         sym_node = self.xml_parser.get_first_child(node, 'sym')
         symbol = self.xml_parser.get_node_data(sym_node)
